package de.hdm_stuttgart.battlearena.Controller;

import de.hdm_stuttgart.battlearena.Controller.Utilities.SceneLoader;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Persistence;
import javafx.beans.binding.Bindings;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.chart.PieChart;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;

import java.net.URL;
import java.util.ResourceBundle;

public class StatisticsController implements Initializable {

    @FXML
    private Text blocks, deaths, gameTime, gamesLost, gamesWon, kills;
    @FXML
    private VBox parent;
    @FXML
    private PieChart kd, wl;
    SceneLoader sceneLoader = new SceneLoader();

    Persistence persistence = Persistence.getInstance();

    public void mainMenuScene() {
        parent.getChildren().clear();
        parent.getChildren().add(sceneLoader.loadScene("MainMenu"));
    }

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        iniPieChartKD();
        iniPieChartWL();
        kd.setLegendVisible(false);
        wl.setLegendVisible(false);
        setLabels();
    }

    private void setLabels() {
        kills.setText(String.valueOf(persistence.getStatistics().getKills()));
        gamesWon.setText(Integer.toString(persistence.getStatistics().getGamesWon()));
        gamesLost.setText(String.valueOf(persistence.getStatistics().getGamesLost()));
        gameTime.setText(Integer.toString(persistence.getStatistics().getGameTimeInHours()));
        deaths.setText(String.valueOf(persistence.getStatistics().getDeaths()));
        blocks.setText(Integer.toString(persistence.getStatistics().getBlocksDestroyed()));
    }

    public void iniPieChartKD() {
        ObservableList<PieChart.Data> pieChartData =
                FXCollections.observableArrayList(
                        new PieChart.Data("Kills", persistence.getStatistics().getKills()),
                        new PieChart.Data("Deaths", persistence.getStatistics().getDeaths()));

        pieChartData.forEach(data ->
                data.nameProperty().bind(
                        Bindings.concat(
                                data.getName(), ": ", data.pieValueProperty()
                        )
                )
        );
        kd.getData().addAll(pieChartData);
    }

    public void iniPieChartWL() {
        ObservableList<PieChart.Data> pieChartData =
                FXCollections.observableArrayList(
                        new PieChart.Data("Wins", persistence.getStatistics().getGamesWon()),
                        new PieChart.Data("Losses", persistence.getStatistics().getGamesLost()));

        pieChartData.forEach(data ->
                data.nameProperty().bind(
                        Bindings.concat(
                                data.getName(), ": ", data.pieValueProperty()
                        )
                )
        );
        wl.getData().addAll(pieChartData);
    }
}

