package de.hdm_stuttgart.battlearena.Model.DataStorage.Classes;

import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.CryptoUtils;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.CryptoException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.SQLException;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.File;
import java.sql.*;
import java.util.ArrayList;

import oracle.jdbc.pool.OracleDataSource;

public class OracleDB implements ISQLDataBase {

    private static String user="";
    private static String password="";
    private static String[] parts= new String[2];
    static File encryptedFile = new File("src\\main\\resources\\database\\OracleDB_logindetails");
    private static final Logger log = LogManager.getLogger(OracleDB.class);

    public static void getLoginData() throws CryptoException {
        parts = CryptoUtils.decrypt(encryptedFile);
        user = parts[0];
        password = parts[1];
        //log.info("OracleDB_logindetails:  user: " + user + "  password: " +  password);
    }

    @Override
    public Connection connect() throws SQLException {
        try {
            getLoginData();
            OracleDataSource ods = new OracleDataSource();
            ods.setURL("jdbc:oracle:thin:@(description= (retry_count=20)(retry_delay=3)(address=(protocol=tcps)(port=1521)(host=adb.eu-frankfurt-1.oraclecloud.com))(connect_data=(service_name=g093caf2cf1fea4_battlearena_high.adb.oraclecloud.com))(security=(ssl_server_dn_match=yes)))");
            ods.setUser(user);
            ods.setPassword(password);
            Connection conn = ods.getConnection();

            log.info("Connecting to SQL database ...");

            return conn;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("SQL connection error! " + e.getMessage());
        }
    }

    @Override
    public ArrayList<MapData> getCoreMaps() throws SQLException {
        try(Connection connection = connect()) {
            ArrayList<MapData> newMaps = new ArrayList<MapData>();
            String sql = "SELECT * FROM battlearenadata.coremaps";
            PreparedStatement stmt = connection.prepareStatement(sql);

            log.info("Sending SQL statement.");
            ResultSet rs = stmt.executeQuery();

            while(rs.next()){
                newMaps.add(new MapData(rs.getString("map_id"), rs.getString("map_name"), rs.getInt("map_width"), rs.getInt("map_height"), rs.getString("map_data")));
            }
            rs.close();
            stmt.close();
            log.info("Core-Maps retrieved successfully from SQL server!");
            return newMaps;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error retrieving Core-Maps from SQL server! " + e.getMessage());
        }
    }

    @Override
    public ArrayList<MapInfo> getCommunityMapsList() throws SQLException {
        try(Connection connection = connect()) {
            ArrayList<MapInfo> tempList = new ArrayList<MapInfo>();
            String sql = "SELECT map_id, map_name, map_width, map_height, map_downloads FROM battlearenadata.communitymaps";
            PreparedStatement stmt = connection.prepareStatement(sql);

            log.info("Sending SQL statement.");
            ResultSet rs = stmt.executeQuery();

            while(rs.next()){
                tempList.add(new MapInfo(rs.getString("map_id"), rs.getString("map_name"), rs.getInt("map_width"), rs.getInt("map_height"), rs.getInt("map_downloads")));
            }
            rs.close();
            stmt.close();
            log.info("Community-Map names retrieved successfully from SQL server!");
            return tempList;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error retrieving Community-Map names from SQL server! " + e.getMessage());
        }
    }

    @Override
    public PlayerStatistics getStatistics(String playerName) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "SELECT games_won, games_lost, kills, deaths, blocks_destroyed, ingame_time FROM battlearenadata.players WHERE player_name = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, playerName);

            log.info("Sending SQL statement.");
            ResultSet rs = stmt.executeQuery();
            if(rs.next() == false){             //if no data matches
                throw new SQLException("No match on SQL database!");
            }
            PlayerStatistics stats = new PlayerStatistics(0, 0, 0, 0, 0, 0);
            while(rs.next()) {
                stats = new PlayerStatistics(rs.getInt("games_won"), rs.getInt("games_lost"), rs.getInt("kills"), rs.getInt("deaths"), rs.getInt("blocks_destroyed"), rs.getInt("ingame_time"));
            }
            rs.close();
            stmt.close();
            log.info("Player statistics retrieved successfully from SQL server!");
            return stats;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error retrieving player statistics from SQL server! " + e.getMessage());
        }
    }

    @Override
    public MapData getCommunityMapByID(String mapID) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "SELECT * FROM battlearenadata.communitymaps WHERE map_ID = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, mapID);

            log.info("Sending SQL statement.");
            ResultSet rs = stmt.executeQuery();
            MapData mapChosen = new MapData("", "",0,0, "");
            if(rs.next() == false){             //if no data matches
                throw new SQLException("No match on SQL database!");
            }
            while(rs.next()) {
                mapChosen = new MapData(rs.getString("map_id"), rs.getString("map_name"), rs.getInt("map_width"), rs.getInt("map_height"), rs.getString("map_data"));
            }
            rs.close();
            stmt.close();
            log.info("Community-Map retrieved successfully from SQL server!");

            String sql2 = "UPDATE battlearenadata.communitymaps SET map_downloads = map_downloads + 1 WHERE map_id = ?";
            PreparedStatement stmt2 = connection.prepareStatement(sql2);
            stmt2.setString(1, mapID);
            stmt2.executeQuery();
            log.info("Sending SQL statement to update Download-Counter for downloaded map!");
            stmt2.close();

            return mapChosen;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error retrieving Community-Map from SQL server! " + e.getMessage());
        }
    }

    @Override
    public String checkCredentials(String playerName) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "SELECT player_pw FROM battlearenadata.players WHERE player_name = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, playerName);

            log.info("Sending SQL statement.");
            ResultSet rs = stmt.executeQuery();

            String password = "";
            while(rs.next()) {
                password = rs.getString("player_pw");
            }
            rs.close();
            stmt.close();
            log.info("Player credentials retrieved successfully from SQL server!");
            return password;
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error retrieving retrieving player credentials from SQL server! " + e.getMessage());
        }
    }

    @Override
    public void uploadCommunityMap(MapData map) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "INSERT INTO battlearenadata.communitymaps (map_id, map_name, map_width, map_height, map_data, map_downloads) VALUES (?, ?, ?, ?, ?, 0)";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, map.getMapID());
            stmt.setString(2, map.getMapName());
            stmt.setInt(3, map.getMapWidth());
            stmt.setInt(4, map.getMapHeight());
            stmt.setString(5, map.getMapData());

            log.info("Sending SQL statement.");
            stmt.executeQuery();

            stmt.close();
            log.info("Community-Map uploaded successfully to SQL server!");
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error uploading created Community-Map to SQL server! " + e.getMessage());
        }
    }

    @Override
    public void createPlayer(String playerName, String playerPW) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "INSERT INTO battlearenadata.players (player_name, player_pw, games_won, games_lost, kills, deaths, blocks_destroyed, ingame_time) VALUES (?, ?, 0, 0, 0, 0, 0, 0)";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, playerName);
            stmt.setString(2, playerPW);

            log.info("Sending SQL statement.");
            stmt.executeQuery();

            stmt.close();
            log.info("Online account created successfully on SQL server!");
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error creating online account on SQL server! " + e.getMessage());
        }
    }

    @Override
    public void updatePlayerStats(PlayerStatistics stats, PlayerAccount account) throws SQLException {
        try(Connection connection = connect()) {
            String sql = "UPDATE battlearenadata.players SET games_won = ?, games_lost = ?, kills = ?, deaths = ?, blocks_destroyed = ?, ingame_time = ? WHERE player_name = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setInt(1, stats.getGamesWon());
            stmt.setInt(2, stats.getGamesLost());
            stmt.setInt(3, stats.getKills());
            stmt.setInt(4, stats.getDeaths());
            stmt.setInt(5, stats.getBlocksDestroyed());
            stmt.setInt(6, stats.getGameTime());
            stmt.setString(7, account.getPlayerName());

            log.info("Sending SQL statement.");
            stmt.executeQuery();

            stmt.close();
            log.info("Player statistics updated successfully on SQL server!");
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error updating player statistics on SQL server! " + e.getMessage());
        }
    }

    @Override
    public void resetPlayerStats(PlayerAccount account) throws SQLException{        //verification of player credentials necessary beforehand (use methode "verifyAccountData" in "Persistence.java)
        try(Connection connection = connect()) {
            String sql = "UPDATE battlearenadata.players SET games_won = 0, games_lost = 0, kills = 0, deaths = 0, blocks_destroyed = 0, ingame_time = 0 WHERE player_name = ?";
            PreparedStatement stmt = connection.prepareStatement(sql);
            stmt.setString(1, account.getPlayerName());

            log.info("Sending SQL statement.");
            stmt.executeQuery();

            stmt.close();
            log.info("Player statistics reset successfully on SQL server!");
        }
        catch(Exception e){
            log.error(e);
            throw new SQLException("Error resetting player statistics on SQL server! " + e.getMessage());
        }
    }

}
